#!/usr/bin/env python
# coding=utf-8
# 
# from: http://th.wikipedia.org/wiki/%E0%B8%A0%E0%B8%B2%E0%B8%A9%E0%B8%B2%E0%B8%9A%E0%B8%B2%E0%B8%A5%E0%B8%B5

import sys
import os

# GLOBAL VARS
lang_rec = [
    #[LANG_NUM, LANG_NAME,  DESC,   SUPPORTED INPUT],
    [1,     'engpali',  'Pali English',     True],
    [101,   'eng',      'Simple English',   False],
    [2,     'thaipali', 'Pali Thai',        True],
    [102,   'thai',     'Simple Thai',      True],
#    [3,     'laopali',  'Pali Lao',         False],    #no font supported
    [103,   'lao',      'Simple Lao',       False],
    [4,     'burmpali', 'Pali Burmese',     True],
]

inlang = outlang = ''
inlang_num = outlang_num = 0
inchar_rec = outchar_rec = p_char_rec = []

#eng
chum = 'ṃ'  # อํ
uchum = u'ṃ'

#thai
cphinthu = '\xe0\xb8\xba'   # อฺ
cyamakkan = '\xe0\xb9\x8e'  # อ๎
cnikkhahit = '\xe0\xb9\x8d' # อํ
cmaihanakat = '\xe0\xb8\xb1'    # อั
csaraa = 'ะ'
coang = 'อ'

ucphinthu = u'\u0e3a'       # อฺ
ucyamakkan = u'\u0e4e'      # อ๎
ucnikkhahit = u'\u0e4d'     # อํ
ucmaihanakat = u'\u0e31'    # อั
ucsaraa = u'ะ'
ucoang = u'อ'

#lao
cmaikan = '\xe0\xba\xb1'    # ອັ
csaraa_lao = '\xe0\xba\xb0' # ອະ

ucmaikan = u'\u0eb1'        # ອັ
ucsaraa_lao = u'\u0eb0'     # ອະ

#burmese
ca_burmese = 'အ'
canusvara = '\xe1\x80\xb6'  # อํ
cvirama = '\xe1\x80\xb9'    # doubler
cga_burmese = '\xe1\x80\x82'    # ค
cda_burmese = '\xe1\x80\x92'    # ท
cpa_burmese = '\xe1\x80\x95'    # ป
cwa_burmese = '\xe1\x80\x9d'    # ว
caa_burmese = '\xe1\x80\xac'    # า
ctallaa_burmese = '\xe1\x80\xab'# tall า
cmedian_wa = '\xe1\x80\xbd'     # /ว

uca_burmese = u'အ'
ucanusvara = '\xe1\x80\xb6'.decode('utf8')  # อํ
ucvirama = '\xe1\x80\xb9'.decode('utf8')    # doubler
ucga_burmese = u'\u1002'    # ค
ucda_burmese = u'\u1012'    # ท
ucpa_burmese = u'\u1015'    # ป
ucwa_burmese = u'\u101d'    # ว
ucaa_burmese = u'\u102c'    # า
uctallaa_burmese = u'\u102b'# tall า
ucmedian_wa = u'\u103d'     # /ว

def cannotopenfile(filename):
    print "Cannot open file %s" % (filename)

def get_chars(lang_num):
    #alphabet: ก k, ข kh, ค g, ฆ gh, ง ṅ, จ c, ฉ ch, ช j, ฌ jh, ญ ñ, ฏ ṭ, ฐ ṭh, ฑ ḍ, ฒ ḍh, ณ ṇ, ต t, ถ th, ท d, ธ dh, น n, ป p, ผ ph, พ b, ภ bh, ม m, ย y, ร r, ล l, ว v, ส s, ห h, ฬ ḷ
    #special_alphabet: อํ ṃ
    #vowel: อะ a, อา ā, อิ i, อี ī, เอ e, อุ u, อู ū, โอ o
    #num: 
    if lang_num == 0:   #psuedo-thaipali
        return [
['ก','ข','ค','ฆ','ง','จ','ฉ','ช','ฌ','ญ','ฏ','ฐ','ฑ','ฒ','ณ','ต','ถ','ท','ธ','น','ป','ผ','พ','ภ','ม','ย','ร','ล','ว','ส','ห','ฬ'],  #alphabet
[cnikkhahit],  #special_alphabet: อํ
['ะ','า','\xe0\xb8\xb4','\xe0\xb8\xb5','เ','\xe0\xb8\xb8','\xe0\xb8\xb9','โ'],  #vowel: 
['๐','๑','๒','๓','๔','๕','๖','๗','๘','๙'],  #num
            ]
    if lang_num == 1:   #englishpali
        return [
['k','kh','g','gh','ṅ','c','ch','j','jh','ñ','ṭ','ṭh','ḍ','ḍh','ṇ','t','th','d','dh','n','p','ph','b','bh','m','y','r','l','v','s','h','ḷ'],    #alphabet
[chum],  #special_alphabet
['a','ā','i','ī','e','u','ū','o'],  #vowel
['0','1','2','3','4','5','6','7','8','9'],  #num
            ]
    if lang_num == 101:   #english
        return [
['k','kh','g','gh','n','c','ch','j','jh','n','t','th','d','dh','n','t','th','d','dh','n','p','ph','b','bh','m','y','r','l','v','s','h','l'],    #alphabet
['m'],  #special_alphabet
['a','a','i','i','e','u','u','o'],  #vowel
['0','1','2','3','4','5','6','7','8','9'],  #num
            ]
    elif lang_num == 2:   #thaipali
        return [
['ก','ข','ค','ฆ','ง','จ','ฉ','ช','ฌ','ญ','ฏ','ฐ','ฑ','ฒ','ณ','ต','ถ','ท','ธ','น','ป','ผ','พ','ภ','ม','ย','ร','ล','ว','ส','ห','ฬ'],  #alphabet
[cnikkhahit],  #special_alphabet: อํ
['','า','\xe0\xb8\xb4','\xe0\xb8\xb5','เ','\xe0\xb8\xb8','\xe0\xb8\xb9','โ'],  #vowel: 
['๐','๑','๒','๓','๔','๕','๖','๗','๘','๙'],  #num
            ]
    elif lang_num == 102: #thai
        return [
['ก','ข','ค','ฆ','ง','จ','ฉ','ช','ฌ','ญ','ฏ','ฐ','ฑ','ฒ','ณ','ต','ถ','ท','ธ','น','ป','ผ','พ','ภ','ม','ย','ร','ล','ว','ส','ห','ฬ'],  #alphabet
['ง'],  #special_alphabet
['ะ','า','\xe0\xb8\xb4','\xe0\xb8\xb5','เ','\xe0\xb8\xb8','\xe0\xb8\xb9','โ'],  #vowel: 
['๐','๑','๒','๓','๔','๕','๖','๗','๘','๙'],  #num
            ]
    elif lang_num == 103: #lao
        return [
['ກ','ຂ','ຄ','ຄ','ງ','ຈ','ສ','ຊ','ຊ','ຍ','ດ','ຖ','ດ','ທ','ນ','ຕ','ຖ','ທ','ທ','ນ','ປ','ຜ','ພ','ພ','ມ','ຢ','ຣ','ລ','ວ','ສ','ຫ','ລ'],  #alphabet
['ງ'],  #special_alphabet
['ະ','າ','\xe0\xba\xb4','\xe0\xba\xb5','ເ','\xe0\xba\xb8','\xe0\xba\xb9','ໂ'],  #vowel: 
['໐','໑','໒','໓','໔','໕','໖','໗','໘','໙'],  #num
            ]
    elif lang_num == 4:   #burmese
        return [
['က','ခ','ဂ','ဃ','င','စ','ဆ','ဇ','ဈ','ည','ဋ','ဌ','ဍ','ဎ','ဏ','တ','ထ','ဒ','ဓ','န','ပ','ဖ','ဗ','ဘ','မ','ယ','ရ','လ','ဝ','သ','ဟ','ဠ'],  #alphabet
[canusvara],  #special_alphabet ္
['','\xe1\x80\xac','\xe1\x80\xad','\xe1\x80\xae','\xe1\x80\xb1','\xe1\x80\xaf','\xe1\x80\xb0','\xe1\x80\xb1\xe1\x80\xac'],  #vowel: 
['၀','၁','၂','၃','၄','၅','၆','၇','၈','၉'],  #num
            ]
    else:
        return [[],[],[],[]] 

def usage(progname,infile,outfile):
    print """\
Usage: %s -f LANG -t LANG %s [%s]""" % (progname,infile,outfile,)
    print """
Options:
    -f, --from: convert from language.
    -t, --to:   convert to language.

Languages:"""
    sample_txt = "อะเสว๎นา จะ พาลานัง, ปัณฑิตานัญจะ เสว๎นา ฯ"
    global inlang_num,inchar_rec,outlang_num,outchar_rec
    inlang_num = 102
    inchar_rec = get_chars(inlang_num)
    for i,j,k,l in lang_rec:
        outlang_num = i
        outchar_rec = get_chars(i)
        txt = convert_pali(sample_txt)
        tj = j+' '*(12-len(j))
        if l:
            tl = '*'
        else:
            tl = ' '
        print """\
%s  %s: %s\t(%s)""" % (tl,tj,k,txt,)
    print """
(* = Supported input language, eg. convert from)"""
    return

def get_argv(parm):
    if parm in sys.argv:
        idx = sys.argv.index(parm)
        sys.argv.pop(idx)
        return sys.argv.pop(idx)
    else:
        return ''

def preprocess(txt,lang_num):   #swap sara
    if lang_num in [2,102,103]:
        utxt = txt.decode('utf8')
        #SWAP SARA
        for fsara in [u'เ',u'โ',u'ເ',u'ໂ']:
            front = u''
            rear = utxt
            while rear and fsara in rear:
                i = rear.index(fsara)
                if len(rear)>i+2 and rear[i+2] in [ucyamakkan,ucphinthu]: #double chars
                    front += rear[:i]+rear[i+1:i+4]+rear[i]
                    rear = rear[i+4:]
                else:
                    front += rear[:i]+rear[i+1]+rear[i]
                    rear = rear[i+2:]
            utxt = front+rear
            
        #FIX SOURCE
        if lang_num == 102:
            clist = [
[u'พรหม',u'พ๎รัห๎ม'],
[u'พราหม',u'พ๎ราห๎ม'],
                ]
            for i,j in clist:
                utxt = utxt.replace(i,j)
        return utxt.encode('utf8')
    else:
        return txt

def pre_convert(txt,lang_num):
    #PRECONVERT: convert any to pseudo-thaipali
    utxt = txt.decode('utf8')
    uallchars = []
    for i in inchar_rec:
        for j in i:
            uallchars.append(j.decode('utf8'))
    uallchars.append(ucoang)
    uallchars.append(u'\u0e31')

    uallalphas = [ i.decode('utf8') for i in inchar_rec[0] ]
    uallalphas.append(ucoang)

    uallsaras = [ i.decode('utf8') for i in inchar_rec[2] ]

    if lang_num == 1: #inlang = thaipali
        #insert phinthu
        #insert อ
        #convert to pseudo-thai

        uallchars.append(ucphinthu)
        uallexcepts = [ i.decode('utf8') for i in inchar_rec[1] ]
        uallexcepts += [ i.decode('utf8') for i in inchar_rec[2] ]
        uallexcepts += [ i.decode('utf8') for i in inchar_rec[3] ]

        #insert phinthu
        u1h = u'kgcjṭḍtdpb'
        front = ''
        rear = utxt
        while rear:
            ispass = False
#            print '0.front=%s, rear=%s' % (front,rear,)
            while rear and (not rear[0].isalpha() or rear[0] in uallexcepts):
                front += rear[0]
                rear = rear[1:]
                ispass = True
            while rear and len(rear)>1 and (rear[1] in uallexcepts or not rear[1].isalpha):
                front += rear[:2]
                rear = rear[2:] 
                ispass = True
            while rear and len(rear)>1 and rear[0] in u1h and rear[1] == u'h':
                front += rear[0]
                rear = rear[1:]
                ispass = True
            if rear and len(rear)>1 and rear[0] == uchum:
                front += rear[0]+ucphinthu
                rear = rear[1:]
                ispass = True
            if rear and len(rear)>2 and rear[0].isalpha() and rear[1].isalpha and \
                    rear[1] not in uallsaras:
                    #rear[1] not in uallsaras + [u'h']:
                front += rear[0]+ucphinthu
                rear = rear[1:]
                ispass = True
            if not ispass:
                front += rear
                rear = u''
#            print '1.front=%s, rear=%s' % (front,rear,)
        utxt = front+rear

        #insert อ
        front = u''
        rear = utxt
        while rear:
            ispass = False
            while rear and not rear[0] in uallchars:
                front += rear[0]
                rear = rear[1:]
                ispass = True
#            print '0.front=%s, rear=%s' % (front,rear,)
            if rear and rear[0] in uallchars:
                if rear[0] in uallsaras:
                    front += ucoang+rear[0]
                    rear = rear[1:]
                else:
                    front += rear[0]
                    rear = rear[1:]
                ispass = True
                while rear and rear[0] in uallchars:
                    front += rear[0]
                    rear = rear[1:]
#            print '1.front=%s, rear=%s' % (front,rear,)
            if not ispass:
                front += rear
                rear = u''
        utxt = front+rear

        #convert to pseudo-thai
        txt = utxt.encode('utf8')
        txt = sub_convert(txt,lang_num,0)
        return txt

    elif lang_num == 2: #inlang = thaipali
        #insert ะ
        uallsaras.append(ucphinthu)

        #insert ะ: นมโ -> นะมโ
        front = u''
        rear = utxt
        while rear:
            while rear and rear[0] not in uallalphas:
                front += rear[0]
                rear = rear[1:]
            while rear and len(rear)>1:
                if rear[0] in uallalphas and rear[1] in uallalphas:
                    front += rear[0]+ucsaraa
                    rear = rear[1:]
                elif rear[1] in uallsaras:
                    front += rear[:2]
                    rear = rear[2:]
                elif rear[0] in uallalphas and not rear[1].isalpha():
                    front += rear[0]+ucsaraa
                    rear = rear[1:]
                else:
                    front += rear[0]
                    rear = rear[1:]
        utxt = front+rear

    elif lang_num == 102: #inlang = thai
        #change trail ง to nikhhahit
        #insert phinthu for spelling-consonant
        #change maihanakat to sara a, plus change yamakkan to phinthu

        #change trail ง to nikhhahit: อัง|อุง|อิง+not_alpha -> change to อะ+nik,อุํ,อิํ
        ucin = u'ง'
        ucout = ucnikkhahit     #อํ
        usara_out = [ i.decode('utf8') for i in inchar_rec[2] ]
        usara_in = [ u'\u0e31' ] + usara_out[1:]    #change อะ -> อั
        l = len(utxt)
        for i in range(len(usara_in)):
            front = u''
            rear = utxt
            while rear and usara_in[i]+ucin in rear:
                idx = rear.index(usara_in[i]+ucin)
                if idx==l-2 or not rear[idx+2].isalpha():
                    front += rear[:idx]+usara_out[i]+ucout
                    rear = rear[idx+2:]
                else:
                    front += rear[:idx+2]
                    rear = rear[idx+2:]
            utxt = front+rear 
#        print "1st utxt=%s" % (utxt,)

        #insert phinthu for spelling-consonant: 'คุตตะระ' -> 'คุตฺ'+'ตะระ', 'สัง ' -> 'สังฺ'+' '
        front = u''
        rear = utxt
        p = 0
        while rear:
            ispass = False
            #not alphabet
            while rear and not rear[0] in uallalphas:
                front += rear[0]
                rear = rear[1:]
                ispass = True
#            print "0.front=%s, rear=%s" % (front,rear[:50],)
            while rear and len(rear)>1 and rear[0] in uallalphas and rear[1] not in uallchars:
                front += rear[0]
                rear = rear[1:]
                ispass = True
#            print "1.front=%s, rear=%s" % (front,rear[:50],)
            #cut double chars: 'อสเ'+'ว๎นา' -> 'อสเว๎'+'นา'
            while rear and len(rear)>1 and rear[1] in [ucyamakkan,ucsaraa]:    #อ๎ yamakar, อะ
                front += rear[:2]
                rear = rear[2:]
                ispass = True
#            print "2.front=%s, rear=%s" % (front,rear[:50],)
            #insert phinthu for spelling-consonant: 'คุตตะระ' -> 'คุตฺ'+'ตะระ', 'สัง ' -> 'สังฺ'+' '
            while rear and len(rear)>1 and rear[1] in usara_in:
                if len(rear)>3 and rear[3] not in usara_in + [ucsaraa] and rear[2].isalpha() and rear[3].isalpha():
                    front += rear[:3]+ucphinthu    #อฺ phinthu
                    rear = rear[3:]
                else:
                    front += rear[:2]
                    rear = rear[2:]
                ispass = True
#            print "3.front=%s, rear=%s" % (front,rear[:50],)
            if not ispass:
                front += rear
                rear = u''
        #change maihanakat to sara a, plus change yamakkan to phinthu
        utxt = front+rear
        utxt = utxt.replace(ucmaihanakat,ucsaraa).replace(ucyamakkan,ucphinthu)

    elif lang_num == 4: #burmese
        #change virama to phinthu,/ว,อ,า
        #insert ะ
        #convert to pseudo-thai
        uallsaras.append(ucphinthu)

        #change virama to phinthu
        rep_list = [
            [ucmedian_wa,ucvirama+u'\u101d'],
            [ucvirama,ucphinthu],
            [uca_burmese,ucoang],
            [uctallaa_burmese,ucaa_burmese],
        ]    
        for i,j in rep_list:
            utxt = utxt.replace(i,j)

        #insert ะ: นมโ -> นะมโ
        front = u''
        rear = utxt
        while rear:
            while rear and rear[0] not in uallalphas:
                front += rear[0]
                rear = rear[1:]
            while rear and len(rear)>1:
                if rear[0] in uallalphas and rear[1] in uallalphas:
                    front += rear[0]+ucsaraa
                    rear = rear[1:]
                elif rear[1] in uallsaras:
                    front += rear[:2]
                    rear = rear[2:]
                elif rear[0] in uallalphas and not rear[1].isalpha():
                    front += rear[0]+ucsaraa
                    rear = rear[1:]
                else:
                    front += rear[0]
                    rear = rear[1:]
        utxt = front+rear

        #convert to pseudo-thai
        txt = utxt.encode('utf8')
        txt = sub_convert(txt,lang_num,0)
        return txt

    txt = utxt.encode('utf8')
    return txt

def sub_convert(txt,inlang_num,outlang_num):
    inchar_rec = get_chars(inlang_num)
    outchar_rec = get_chars(outlang_num)
    for i in range(len(inchar_rec)):
        conv_pair = []
        for j in range(len(inchar_rec[i])):
            conv_pair.append([len(inchar_rec[i][j]),inchar_rec[i][j],outchar_rec[i][j]])
        conv_pair.sort(reverse=True)
        for t,x,y in conv_pair:
            if x and x != y:
                while x in txt:
                    txt = txt.replace(x,y)
    return txt

def do_convert(txt,lang_num):
    txt = sub_convert(txt,0,lang_num)
    return txt

def postprocess(txt,lang_num):
    if lang_num in [1,101]:   #eng
        clean_list = [cphinthu,cyamakkan,'อ']
        for i in clean_list:
            txt = txt.replace(i,'')
    elif lang_num in [2,102,103]:   #thai-lao
        utxt = txt.decode('utf8')
        #change ะ to maihanakat
        if lang_num in [102,103]:
            front = u''
            rear = utxt
            while rear and (ucsaraa in rear or ucsaraa_lao in rear):
                if lang_num == 102:
                    i = rear.index(ucsaraa)
                elif lang_num == 103:
                    i = rear.index(ucsaraa_lao)
#                print "0.front=%s, rear[i+1]=%s, rear=%s" % (front, rear[i+1], rear,)
                if len(rear)>i+2 and rear[i+2] in [ucphinthu]:
                    if lang_num == 102:
                        front += rear[:i]+ucmaihanakat+rear[i+1:i+2]
                        rear = rear[i+2:]
                    elif lang_num == 103:
                        front += rear[:i]+ucmaikan+rear[i+1:i+2]
                        rear = rear[i+3:]
                else:
                    front += rear[:i+1]
                    rear = rear[i+1:]
#                print "1.front=%s, rear=%s" % (front, rear,)
            utxt = front+rear
            #fix unknown อะง -> ัอัง
            if lang_num == 102:
                rep_list = [\
                    [u'\u0e30\u0e07',u'\u0e31\u0e07'],  #อะง -> ัอัง
                    ]
                for i,j in rep_list:
                    utxt = utxt.replace(i,j)
            elif lang_num == 103:
                utxt = utxt.replace(u'\u0eb0\u0e87',u'\u0eb1\u0e87')    #อะง -> ัอัง

        cb2_thai_list = [u'ย',u'ร',u'ว',u'ล',u'ฬ',u'ห']  #double second chars
        cb2_lao_list = [u'ຢ',u'ຣ',u'ວ',u'ລ',u'ຫ']  #double second chars
        cb2_list = cb2_thai_list + cb2_lao_list

        #strip yamakkan for not double chars: ปัณ๎ฑิตา -> ปัณฑิตา
        if lang_num in [102]:
            utxt = utxt.replace(ucphinthu,ucyamakkan)
            front = u''
            rear = utxt
            while rear and ucyamakkan in rear:
                i = rear.index(ucyamakkan)
#                print "0.front=%s, rear[i-1]=%s,rear[i+1]=%s, rear=%s" % (front, rear[i-1],rear[i+1], rear,)
                if len(rear)>i+1 and ((rear[i+1] not in cb2_list+[u'น'] and rear[i-1] not in u'ห') or rear[i-1] == rear[i+1] or rear[i-1] in u'ง'):
                    front += rear[:i]
                    rear = rear[i+1:]
                else:
                    front += rear[:i+1]
                    rear = rear[i+1:]
#                print "1.front=%s, rear=%s" % (front, rear,)
            utxt = front+rear

        #SWAP SARA
        for fsara in [u'เ',u'โ',u'ເ',u'ໂ']:
            front = u''
            rear = utxt
            while rear and fsara in rear:
                i = rear.index(fsara)
                #check double chars: นิโค๎รธะ, วิรุโฬ๎ห
                if len(rear)>3 and rear[i-1] in cb2_list and rear[i-2] in [ucyamakkan,ucphinthu]:
                    front += rear[:i-3]+rear[i]+rear[i-3:i]
                    rear = rear[i+1:]
                else:
                    front += rear[:i-1]+rear[i]+rear[i-1]
                    rear = rear[i+1:]
                ##not check: -> นิค๎โรธะ, วิรุฬ๎โห
                #front += rear[:i-1]+rear[i]+rear[i-1]
                #rear = rear[i+1:]
            utxt = front+rear

        if lang_num in [103]:
            #strip phinthu for lao
            utxt = utxt.replace(ucphinthu,u'')
            #paiyalnoi
            utxt = utxt.replace(u'ฯ',u'ຯ')

        txt = utxt.encode('utf8')

    elif lang_num in [4]:   #burmese
        rep_list = [\
            [coang, ca_burmese],
            [cphinthu,  cvirama],
            ]
        for i,j in rep_list:
            txt = txt.replace(i,j)

        utxt = txt.decode('utf8')
        #clean virama for doubled chars
        front = u''
        rear = utxt
        while rear and ucvirama in rear:
            i = rear.index(ucvirama)
            if i>2 and rear[i+2] == ucvirama:
                front += rear[:i+2]
                rear = rear[i+3:] 
            else:
                front += rear[:i+2]
                rear = rear[i+2:]
        utxt = front+rear
        for char in u'ဂဒပဝ':
            front = u''
            rear = utxt
            c = char+ucaa_burmese
            d = char+uctallaa_burmese
            while rear and c+ucaa_burmese in rear:
                i = rear.index(c)
                if i>0 and rear[i-1] != ucvirama:
                    front += rear[:i]+d
                    rear = rear[i+2:]
                else:
                    front += rear[:i+2]
                    rear = rear[i+2:]
        utxt = front+rear

        #change exception virama+wa -> median_wa : //+ว -> /ว
        rep_list = [ [ucvirama+ucwa_burmese, ucmedian_wa], ]  # //+ว -> /ว
        for i,j in rep_list:
            utxt = utxt.replace(i,j)

        #change aa to tall aa, ??? except doubled char, will use previous char instead
        for c in [ucga_burmese,ucda_burmese,ucpa_burmese,ucwa_burmese]: #คทปว
            front = u''
            rear = utxt
            while rear and c in rear:
                i = rear.index(c)
#                print "0. front=%s, rear=%s" % (front,rear,)
                if len(rear)>i+2 and rear[i+1]==ucvirama and rear[i+3]==ucaa_burmese:
                    front += rear[:i+3]+uctallaa_burmese
                    rear = rear[i+4:]
                elif len(rear)>i+1 and rear[i+1]==ucmedian_wa and rear[i+2]==ucaa_burmese:
                    front += rear[:i+2]+uctallaa_burmese
                    rear = rear[i+3:]
                elif len(rear)>i and rear[i+1]==ucaa_burmese:
                    front += rear[:i+1]+uctallaa_burmese
                    rear = rear[i+2:]
                else:
                    front += rear[:i+1]
                    rear = rear[i+1:]
#                print "1. front=%s, rear=%s" % (front,rear,)
            utxt = front+rear
        txt = utxt.encode('utf8')
    return txt

def convert_pali(txt):
    #PREPROCESS:    swap front sara เ,โ
    #PRE_CONVERT:   convert to pseudo-thaipali
    #DO_CONVERT:    convert from pseduo-thaipali to dest
    #POSTPROCESS:   swap front sara back
    txt = preprocess(txt,inlang_num)
#    print "after preprocess txt=%s" % (txt,)
    txt = pre_convert(txt,inlang_num)
#    print "after pre_convert txt=%s" % (txt,)
    txt = do_convert(txt,outlang_num)
#    print "after do_convert txt=%s" % (txt,)
    txt = postprocess(txt,outlang_num)
#    print "after postprocesstxt=%s" % (txt,)
    return txt

if __name__=="__main__":
    progname=os.path.basename(sys.argv[0])
    modl = []

    inlang = outlang = ''

    lang_num_list = [i for i,j,k,l in lang_rec]
    lang_list = [j for i,j,k,l in lang_rec]

    for i in ['-f','--from']:
        if i in sys.argv:
            inlang = get_argv(i)
            break

    for i in ['-t','--to']:
        if i in sys.argv:
            outlang = get_argv(i)
            break

    if (not (inlang and outlang and inlang in lang_list and outlang in lang_list)) or inlang==outlang:
        usage(progname,'INFILE','OUTFILE')
        sys.exit(1)

    p_char_rec = get_chars(0)
    inlang_num = lang_rec[lang_list.index(inlang)][0]
    inchar_rec = get_chars(inlang_num)
    outlang_num = lang_rec[lang_list.index(outlang)][0]
    outchar_rec = get_chars(outlang_num)

    supported_inlang_num = [i for i,j,k,l in lang_rec if l]
    if inlang_num not in supported_inlang_num:
        l = [ lang_rec[lang_num_list.index(i)][1] for i in supported_inlang_num ]
        print """\
Not supported.
Supported input language are: %s.
""" % (','.join(l))
        usage(progname,'INFILE','OUTFILE')
        sys.exit(1)
    try:
        infile = sys.argv[1]
    except:
        usage(progname,'INFILE','OUTFILE')
        sys.exit(1)

    try:
        outfile = sys.argv[2]
    except:
        t1,t2 = os.path.splitext(os.path.basename(infile))
        outfile = t1+'-'+outlang+t2

    try:
        fsold=open(infile)
        txt = fsold.read()
        fsold.close()
    except:
        cannotopenfile(infile)
        usage(progname,'INFILE','OUTFILE')
        sys.exit(1)

    txt = convert_pali(txt)
    f = open(outfile,'w')
    f.write(txt)
    f.close()

    print "Convert from %s (%s) to %s (%s) success." % (inlang,infile,outlang,outfile)
